#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings as django_settings
from django.core.exceptions import ImproperlyConfigured

from .utils import SettingsBase, import_object


class UrlSSOSettings(SettingsBase):
    """ Settings specific to django-url-sso. """
    settings_prefix = 'URL_SSO'

    DEFAULT_REQUEST_TIMEOUT = 5

    @property
    def PLUGINS(self):
        """ Instantiate URL SSO plugins from import path. """

        url_sso_plugins = getattr(
            django_settings, "URL_SSO_PLUGINS", []
        )

        if url_sso_plugins:
            try:
                url_sso_plugins = map(import_object, url_sso_plugins)

            except Exception as e:
                # Catch ImportError and other exceptions too
                # (e.g. user sets setting to an integer)
                raise ImproperlyConfigured(
                    "Error while importing setting "
                    "URL_SSO_PLUGINS %r: %s" % (
                        url_sso_plugins, e
                    )
                )

        return url_sso_plugins

url_sso_settings = UrlSSOSettings()



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Test utils """

from django.test.client import RequestFactory
from django.contrib.auth.models import User


class UserTestMixin(object):
    """ TestCase mixin for tests requiring a logged in user. """

    def setUp(self):
        """ Create a local user and login """

        self.user = User.objects.create_user(
            username='john',
            email='john_lennon@beatles.com',
            password='top_secret'
        )

        login_success = self.client.login(
            username='john', password='top_secret'
        )
        assert login_success, 'Test login failed. Tests are broken.'

        # Call super
        super(UserTestMixin, self).setUp()


class RequestTestMixin(object):
    """ TestCase mixin for tests requiring a request. """

    def setUp(self):
        """ Create request object for '/' URL """

        self.factory = RequestFactory()
        self.request = self.factory.get('/')

        # Call super
        super(RequestTestMixin, self).setUp()



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import patch
from httmock import urlmatch, HTTMock

from django.core import cache

from django.test import TestCase
from django.test.utils import override_settings

from url_sso.context_processors import login_urls
from url_sso.tests.utils import RequestTestMixin, UserTestMixin
from url_sso.exceptions import RequestKeyException

from url_sso.plugins import intershift
from url_sso.plugins.intershift import intershift_plugin


# Setup sensible test settings
intershift_settings = {
    # Secret key as specified by Intershift
    'secret': '12345678',
    # Sites enabled for SSO
    'sites': {
        'site1': {
            # Users never have access to site1
            'has_access': lambda request: False,
            'url': 'https://customer1.intershift.nl/site1/cust/singlesignon.asp',
        },
        'site2': {
            # Users always have acces to site2
            'has_access': lambda request: True,
            'url': 'https://customer1.intershift.nl/site2/cust/singlesignon.asp',
        },
        'site3': {
            # No explicit access rules; same result as site2
            'url': 'https://customer1.intershift.nl/site3/cust/singlesignon.asp',
        },
    },
    # Key expiration in seconds, use one day here
    'key_expiration': 86400
}

sso_settings = {
    'URL_SSO_PLUGINS': ['url_sso.plugins.intershift.intershift_plugin'],
    'URL_SSO_INTERSHIFT': intershift_settings
}


@override_settings(**sso_settings)
class IntershiftTests(RequestTestMixin, UserTestMixin, TestCase):
    """ Tests for Intershift SSO. """

    test_key = 'BOGUSKEY'
    test_xml = '<?XML VERSION="1.0" Encoding="UTF-8"?><xml><key value="BOGUSKEY" /></xml>'
    test_login_url = 'https://customer1.intershift.nl/site1/cust/singlesignon.asp?user=john&key=BOGUSKEY'
    test_login_urls = {
        'INTERSHIFT_SITE2_SSO_URL': 'https://customer1.intershift.nl/site2/cust/singlesignon.asp?user=john&key=BOGUSKEY',
        'INTERSHIFT_SITE3_SSO_URL': 'https://customer1.intershift.nl/site3/cust/singlesignon.asp?user=john&key=BOGUSKEY'
    }

    def setUp(self):
        # Setup local memory cache for tests
        # Source: http://www.2general.com/blog/2012/08/09/changing_django_cache_backend_between_test_cases.html
        self.locmem_cache = cache.get_cache(
            'django.core.cache.backends.locmem.LocMemCache')
        self.locmem_cache.clear()

        # Note: the reason this code is not generalized is because the
        # module is passed as an argument to patch()
        self.cache_patch = patch.object(
            intershift, 'cache', self.locmem_cache
        )
        self.cache_patch.start()

        super(IntershiftTests, self).setUp()

    def tearDown(self):
        self.cache_patch.stop()

    def test_get_site_url(self):
        """ Test _get_site_url() """

        site_url = intershift_plugin._get_site_url('site1')

        self.assertEquals(
            site_url,
            intershift_settings['sites']['site1']['url']
        )

    def test_parse_login_key(self):
        """ Test _parse_login_key() """

        login_key = intershift_plugin._parse_login_key(self.test_xml)

        self.assertEquals(login_key, self.test_key)

    def test_parse_login_key_exceptions(self):
        """ Test for exception on invalid key """

        # Invalid XML should yield exception
        invalid_xml = 'banana'
        self.assertRaises(
            RequestKeyException,
            lambda: intershift_plugin._parse_login_key(invalid_xml)
        )

        # Empty key should raise an exception as well
        empty_key = \
            '<?XML VERSION="1.0" Encoding="UTF-8"?><xml><key value="" /></xml>'
        self.assertRaises(
            RequestKeyException,
            lambda: intershift_plugin._parse_login_key(empty_key)
        )

    def test_request_login_key(self):
        """ Test _request_login_key() """

        @urlmatch(
            scheme='https',
            netloc='customer1.intershift.nl',
            path='/site1/cust/singlesignon.asp'
        )
        def key_mock(url, request):
            return self.test_xml

        with HTTMock(key_mock):
            login_key = intershift_plugin._request_login_key(
                site_name='site1', username=self.user.username
            )

        self.assertEquals(login_key, self.test_key)

    def test_request_login_500(self):
        """ Test exceptions on 500 in _request_login_key() """

        def mock_servererror(url, request):
            return {'status_code': 500}

        with HTTMock(mock_servererror):
            self.assertRaises(
                RequestKeyException,
                lambda: intershift_plugin._request_login_key(
                    'site1', self.user.username
                )
            )

    def test_request_login_empty(self):
        """ Test exception on empty answer from _request_login_key() """

        def mock_empty(url, request):
            return ''

        with HTTMock(mock_empty):
            self.assertRaises(
                RequestKeyException,
                lambda: intershift_plugin._request_login_key(
                    'site1', self.user.username
                )
            )

    def test_generate_login_url(self):
        """ Test _generate_login_url() """

        def key_mock(url, request):
            return self.test_xml

        with HTTMock(key_mock):
            login_url = intershift_plugin._generate_login_url(
                'site1', self.user.username
            )

        self.assertEquals(
            login_url,
            self.test_login_url
        )

    def test_generate_login_url_cache(self):
        """ Test caching for _generate_login_url() """

        def key_mock(url, request):
            return self.test_xml

        # Populate cache
        with HTTMock(key_mock):
            intershift_plugin._generate_login_url(
                'site1', self.user.username
            )

        def fail_mock(url, request):
            self.fail('Request should not be fired when using cache.')

        with HTTMock(fail_mock):
            login_url = intershift_plugin._generate_login_url(
                'site1', self.user.username
            )

        self.assertEquals(
            login_url,
            self.test_login_url
        )

    def test_get_login_url_key(self):
        """ Test _get_login_url_key() """

        self.assertEquals(
            intershift_plugin._get_login_url_key('site1'),
            'INTERSHIFT_SITE1_SSO_URL'
        )

    def test_get_login_url(self):
        """ Test _get_login_url() """

        def key_mock(url, request):
            return self.test_xml

        with HTTMock(key_mock):
            login_url = intershift_plugin._get_login_url('site1', self.user)

        self.assertEquals(login_url, self.test_login_url)

    def test_get_login_urls(self):
        """ Test _get_login_urls() """

        # Make sure user is set on the request
        self.request.user = self.user

        def key_mock(url, request):
            return self.test_xml

        with HTTMock(key_mock):
            urls = intershift_plugin.get_login_urls(self.request)

        # Note: site1 is setup with has_access returning False so a login URL
        # should not be available. Hence, only site2 and site4 are available.
        self.assertEquals(
            urls,
            self.test_login_urls
        )

    def test_integration(self):
        """ Test integration with login_urls() RequestContextProcessor """

        # Make sure user is set on the request
        self.request.user = self.user

        def key_mock(url, request):
            return self.test_xml

        with HTTMock(key_mock):
            context = login_urls(self.request)

            self.assertEquals(
                context,
                self.test_login_urls
            )



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" SSO for Intershift http://www.intershift.nl/ """

import urllib
import sys

from lxml import etree

from django.core.cache import cache

from url_sso.plugins.base import SSOPluginBase
from url_sso.exceptions import RequestKeyException


class IntershiftPlugin(SSOPluginBase):
    settings_name = 'INTERSHIFT'

    def _get_site_url(self, site_name):
        """ Util method to get site url from name. """

        settings = self.get_settings()

        assert 'sites' in settings
        assert site_name in settings['sites'].keys()
        assert 'url' in settings['sites'][site_name]

        return settings['sites'][site_name]['url']

    def _parse_login_key(self, data):
        """ Parse returned (broken) XML and return loginkey. """

        # Use parse with recovery enabled
        parser = etree.XMLParser(recover=True)

        try:
            root = etree.fromstring(data, parser)
            key_element = root.find('key')
            value = key_element.get('value')
        except Exception, e:
            # Raise exception, retaining original traceback
            traceback = sys.exc_info()[2]
            raise RequestKeyException('Error parsing XML: %s' % e), \
                None, traceback

        if not value:
            raise RequestKeyException('No value found in login key response.')

        return value

    def _request_login_key(self, site_name, username):
        """ Request and return login URL for a particular site and user. """

        settings = self.get_settings()

        assert 'secret' in settings

        # Send out request
        r = self.get_url(
            self._get_site_url(site_name),
            params={
                'user': username,
                'secret': settings['secret']
            }
        )

        if not r.status_code == 200:
            raise RequestKeyException(
                'Request returned an error status.'
            )

        if not r.content:
            raise RequestKeyException(
                'Login key request returned no content.'
            )

        return self._parse_login_key(r.content)

    def _get_cache_key(self, site_name, username):
        """ Return a sensible cache key for site and username """
        return 'intershift_sso_{site}_{user}'.format(
            site=site_name, user=username
        )

    def _generate_login_url(self, site_name, username):
        """ Generate and return a login URL from site_name and username. """

        settings = self.get_settings()

        cache_key = self._get_cache_key(site_name, username)
        cache_timeout = settings['key_expiration']

        # Attempt to get key from cache
        login_url = cache.get(cache_key)
        if not login_url:
            # Fetch a login key
            login_key = self._request_login_key(site_name, username)

            # Get site URL
            site_url = self._get_site_url(site_name)

            params = urllib.urlencode({
                'user': username,
                'key': login_key
            })

            # Generate login url
            login_url = '{url}?{params}'.format(url=site_url, params=params)

            # Store cached value for later use
            cache.set(cache_key, login_url, cache_timeout)

        return login_url

    def _get_login_url_key(self, site_name):
        """ Utility method returning context key for login URL. """

        assert site_name

        return 'INTERSHIFT_{site}_SSO_URL'.format(
            site=site_name.upper()
        )

    def _get_login_url(self, site_name, user):
        """ Return login URL for a particular configured site and user. """

        assert user.is_authenticated(), 'User not authenticated.'

        return self._generate_login_url(site_name, user.username)

    def get_login_urls(self, request):
        """ Return login URLs for all configured sites. """

        settings = self.get_settings()

        login_urls = {}

        # Only perform for logged in users
        if request.user.is_authenticated():
            for site_name, site in settings['sites'].iteritems():

                # Determine whether user has access rights
                has_access = site.get('has_access', lambda request: True)
                if not has_access(request):
                    # has_access(request) is defined and user does not have
                    # access to this site - skip.
                    continue

                # Get login URL for site and request (user)
                login_url = self._get_login_url(site_name, request.user)

                # Generate name for URL in context
                login_url_key = self._get_login_url_key(site_name)

                # Add key and URL to login_urls dictionary
                assert not login_url_key in login_urls, 'Duplicate URL.'
                login_urls[login_url_key] = login_url

        return login_urls

# Instantiate singleton
intershift_plugin = IntershiftPlugin()



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from httmock import HTTMock

from django.test import TestCase
from django.test.utils import override_settings

from ..mock_plugins import mock_plugin_one

from url_sso.exceptions import RequestKeyException


class BaseTests(TestCase):
    """ Test functionality from SSOPluginBase """

    @override_settings(URL_SSO_ONE={'TEST': True})
    def test_get_settings(self):
        """ Test _get_settings() """

        settings = mock_plugin_one.get_settings()

        self.assertEquals(settings, {'TEST': True})

    def test_get_url(self):
        """ Test get_url() """

        def success_mock(url, request):
            return 'success'

        with HTTMock(success_mock):
            r = mock_plugin_one.get_url('https://www.bogus.com/')

            self.assertEquals(r.status_code, 200)
            self.assertEquals(r.content, 'success')

    def test_get_url_exception(self):
        """ Test get_url() exception """

        # Attempt a failure (hard to mock)
        self.assertRaises(
            RequestKeyException,
            lambda: mock_plugin_one.get_url('https://weirddomainnamethatdoesnotexist3423423432.com/')
        )



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from mock import Mock, patch
from httmock import HTTMock

from django.core import cache

from django.test import TestCase
from django.test.utils import override_settings

from url_sso.plugins import iprova
from url_sso.plugins.iprova import iprova_plugin
from url_sso.tests.utils import RequestTestMixin, UserTestMixin

# Setup test settings
iprova_settings = {
    # Service root URL
    'root_url': 'http://intranet.organisation.com/',

    # Services available for SSO
    'services': ('management', 'idocument', 'iportal', 'itask'),

    # Key expiration in seconds, use one hour here
    'key_expiration': 3600,

    'application_id': 'SharepointIntranet_Production'
}

sso_settings = {
    'URL_SSO_PLUGINS': ['url_sso.plugins.iprova.iprova_plugin'],
    'URL_SSO_IPROVA': iprova_settings
}


@override_settings(**sso_settings)
class iProvaTests(RequestTestMixin, UserTestMixin, TestCase):
    """ Tests for iProva SSO """

    test_token = '3f5c99f7d8214862afa8c27826b78e14'

    def setUp(self):
        # Setup local memory cache for tests
        # Source: http://www.2general.com/blog/2012/08/09/changing_django_cache_backend_between_test_cases.html
        self.locmem_cache = cache.get_cache(
            'django.core.cache.backends.locmem.LocMemCache')
        self.locmem_cache.clear()

        # Note: the reason this code is not generalized is because the
        # module is passed as an argument to patch()
        self.cache_patch = patch.object(
            iprova, 'cache', self.locmem_cache
        )
        self.cache_patch.start()

        # Setup test WSDL for mock responses
        directory = os.path.join(os.path.dirname(__file__), '..', 'data')

        self.test_wsdl = open(os.path.join(
            directory, 'iprova_usermanagement_wsdl.xml'
        )).read()

        self.test_response = open(os.path.join(
            directory, 'iprova_token_response.xml'
        )).read()

        super(iProvaTests, self).setUp()

    def tearDown(self):
        self.cache_patch.stop()

    def test_get_webservice(self):
        """ Use test WSDL to check whether or not SUDS is broken. """

        def wsdl_mock(url, request):
            self.assertEquals(
                url.geturl(),
                'http://intranet.organisation.com/'
                'Management/Webservices/UserManagementAPI.asmx?WSDL'
            )

            return self.test_wsdl

        with HTTMock(wsdl_mock):
            service = iprova_plugin._get_webservice()

        # Assert the used method is available in WSDL
        self.assertTrue(hasattr(service, 'GetTokenForUser'))

        with HTTMock(
            lambda url, request: self.test_response.format(token='test_token')
        ):
            answer = service.GetTokenForUser(
                strTrustedApplicationID=iprova_settings['application_id'],
                strLoginCode='test_user'
            )

        self.assertEquals(answer, 'test_token')

    def test_get_webservice_cache(self):
        """ Test caching for WSDL files """

        with HTTMock(lambda url, request: self.test_wsdl):
            service = iprova_plugin._get_webservice()

        # Repeating same request should not actually fire
        with HTTMock(lambda url, request: self.fail('Expected one request.')):
            iprova_plugin._get_webservice()

        with HTTMock(
            lambda url, request: self.test_response.format(token='test_token_2')
        ):
            answer = service.GetTokenForUser(
                strTrustedApplicationID=iprova_settings['application_id'],
                strLoginCode='test_user'
            )

        self.assertEquals(answer, 'test_token_2')

        with HTTMock(
            lambda url, request: self.test_response.format(token='test_token_3')
        ):
            answer = service.GetTokenForUser(
                strTrustedApplicationID=iprova_settings['application_id'],
                strLoginCode='test_user'
            )

        self.assertEquals(answer, 'test_token_3')

    @patch('url_sso.plugins.iprova.iprova_plugin._get_webservice')
    def test_request_token(self, mock_method):
        """ Test _request_token() """

        mock_soap_call = Mock()

        class MockService(object):
            GetTokenForUser = mock_soap_call
        mock_soap_call.return_value = self.test_token

        mock_method.return_value = MockService

        token = iprova_plugin._request_token('test_user')

        mock_soap_call.assert_called_once_with(
            strTrustedApplicationID=iprova_settings['application_id'],
            strLoginCode='test_user'
        )

        self.assertEquals(token, self.test_token)

    def test_get_cache_key(self):
        """ Test _get_cache_key() """

        cache_key = iprova_plugin._get_cache_key('my_name')
        self.assertEquals(cache_key, 'iprova_sso_my_name')

    @patch('url_sso.plugins.iprova.iprova_plugin._request_token')
    def test_get_login_token(self, mock_method):
        """ Test _get_login_token() """

        mock_method.return_value = self.test_token

        # Call once, populate cache
        token = iprova_plugin._get_login_token('test_user')
        self.assertEquals(token, self.test_token)

        # Call once more
        iprova_plugin._get_login_token('test_user')

        # Due to cache, _request_token should have only been called once
        mock_method.assert_called_once_with('test_user')

    def test_generate_login_url(self):
        """ Test _generate_login_url() """

        login_url = iprova_plugin._generate_login_url(
            'http://intranet.organisation.com/iprova/',
            self.test_token
        )

        self.assertEquals(
            login_url,
            'http://intranet.organisation.com/iprova/?token=' + self.test_token
        )

    @patch('url_sso.plugins.iprova.iprova_plugin._get_login_token')
    def test_get_login_urls(self, mock_method):
        """ Test get_login_urls() """

        # Make sure user is set on the request
        self.request.user = self.user

        mock_method.return_value = self.test_token

        urls = iprova_plugin.get_login_urls(self.request)

        self.assertEquals(urls, {
            'IPROVA_MANAGEMENT_SSO_URL':
                'http://intranet.organisation.com/management/?token=' + self.test_token,
            'IPROVA_IDOCUMENT_SSO_URL':
                'http://intranet.organisation.com/idocument/?token=' + self.test_token,
            'IPROVA_IPORTAL_SSO_URL':
                'http://intranet.organisation.com/iportal/?token=' + self.test_token,
            'IPROVA_ITASK_SSO_URL':
                'http://intranet.organisation.com/itask/?token=' + self.test_token
        })

        mock_method.assert_called_once_with(self.user.username)

    @patch('url_sso.plugins.iprova.iprova_plugin._get_login_token')
    def test_has_access(self, mock_method):
        """ Test get_login_urls() with has_access = False """

        # Make sure user is set on the request
        self.request.user = self.user

        mock_method.return_value = self.test_token

        local_settings = sso_settings.copy()
        local_settings['URL_SSO_IPROVA']['has_access'] = \
            lambda request, service: False

        with override_settings(**local_settings):
            urls = iprova_plugin.get_login_urls(self.request)

        self.assertEquals(urls, {})

        # No request should have been sent out
        self.assertFalse(mock_method.called)

        # Now test with just one service allowed
        local_settings['URL_SSO_IPROVA']['has_access'] = \
            lambda request, service: service == 'iportal'

        with override_settings(**local_settings):
            urls = iprova_plugin.get_login_urls(self.request)

        self.assertEquals(urls, {
            'IPROVA_IPORTAL_SSO_URL':
                'http://intranet.organisation.com/iportal/?token=' + self.test_token,
        })

        mock_method.assert_called_once_with(self.user.username)



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Mock plugins used for testing. """

from url_sso.exceptions import RequestKeyException
from url_sso.plugins.base import SSOPluginBase


class MockPluginOne(SSOPluginBase):
    settings_name = 'ONE'

    bogus_dict = {
        'MY_URL': 'https://www.bogus.com/some_token'
    }

    def get_login_urls(self, request):
        return self.bogus_dict


class MockPluginException(SSOPluginBase):
    settings_name = 'EXCEPTION'

    def get_login_urls(self, request):
        raise RequestKeyException('bananas')


class MockPluginTwo(SSOPluginBase):
    settings_name = 'TWO'

    bogus_dict = {
        'OTHER_URL': 'https://www.bogus.com/other_token'
    }

    def get_login_urls(self, request):
        return self.bogus_dict


# Instantiate singletons
mock_plugin_one = MockPluginOne()
mock_plugin_exception = MockPluginException()
mock_plugin_two = MockPluginTwo()



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
logger = logging.getLogger(__name__)

from .settings import url_sso_settings
from .exceptions import RequestKeyException


def login_urls(request):
    """
    Make sure SSO login URL's are available in the template context.
    """

    login_urls = {}
    for sso_plugin in url_sso_settings.PLUGINS:
        assert hasattr(sso_plugin, 'get_login_urls'), \
            'No get_login_urls in SSO plugin.'

        try:
            new_login_urls = sso_plugin.get_login_urls(request)
        except RequestKeyException:
            # Log the stack trace but don't make the context processor fail
            logger.exception(
                'Error requesting login key for %s', sso_plugin
            )

            # Continue to next SSO plugin
            continue

        assert not filter(lambda x: x in login_urls, new_login_urls), \
            'Login URL already present.'

        # Add new URL's
        login_urls.update(new_login_urls)

    return login_urls



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Common tests seperate from plugins. """

from django.test import TestCase

from url_sso.context_processors import login_urls

from .mock_plugins import mock_plugin_one

from .utils import RequestTestMixin


class ContextProcessorTests(RequestTestMixin, TestCase):
    """ Test for context processor """

    def test_no_plugins(self):
        """ Test login URL's when no plugins are configured """

        with self.settings(URL_SSO_PLUGINS=[]):
            self.assertEquals(login_urls(self.request), {})

    def test_mock_plugin(self):
        """ Test with a mock plugin """

        sso_plugins = ['url_sso.tests.mock_plugins.mock_plugin_one']

        with self.settings(URL_SSO_PLUGINS=sso_plugins):

            self.assertEquals(
                login_urls(self.request),
                mock_plugin_one.bogus_dict
            )

    def test_exception(self):
        """ Test with a mock plugin raising a RequestKeyException """

        sso_plugins = ['url_sso.tests.mock_plugins.mock_plugin_exception']

        with self.settings(URL_SSO_PLUGINS=sso_plugins):
            # This should yield no exception but return an empty dict
            self.assertEquals(
                login_urls(self.request),
                {}
            )

    def test_two_plugins(self):
        """ Test with two bogus plugins """

        sso_plugins = [
            'url_sso.tests.mock_plugins.mock_plugin_one',
            'url_sso.tests.mock_plugins.mock_plugin_two'
        ]

        with self.settings(URL_SSO_PLUGINS=sso_plugins):

            self.assertEquals(
                login_urls(self.request),
                {
                    'MY_URL': 'https://www.bogus.com/some_token',
                    'OTHER_URL': 'https://www.bogus.com/other_token'
                }
            )



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings as django_settings
from django.utils.importlib import import_module
from django.core.cache import cache

from suds.cache import Cache


class Singleton(type):
    """
    Singleton metaclass.
    Source:
    http://stackoverflow.com/questions/6760685/creating-a-singleton-in-python
    """
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(
                *args, **kwargs
            )

        return cls._instances[cls]


class SettingsBase(object):
    """
    A settings object that proxies settings and handles defaults, inspired
    by `django-appconf` and the way it works  in `django-rest-framework`.

    By default, a single instance of this class is created as `<app>_settings`,
    from which `<APP>_SETTING_NAME` can be accessed as `SETTING_NAME`, i.e.::

        from myapp.settings import myapp_settings

        if myapp_settings.SETTING_NAME:
            # DO FUNKY DANCE

    If a setting has not been explicitly defined in Django's settings, defaults
    can be specified as `DEFAULT_SETTING_NAME` class variable or property.
    """

    __metaclass__ = Singleton

    def __init__(self):
        """
        Assert app-specific prefix.
        """
        assert hasattr(self, 'settings_prefix'), 'No prefix specified.'

    def __getattr__(self, attr):
        """
        Return Django setting `PREFIX_SETTING` if explicitly specified,
        otherwise return `PREFIX_SETTING_DEFAULT` if specified.
        """

        if attr.isupper():
            # Require settings to have uppercase characters

            try:
                setting = getattr(
                    django_settings,
                    '%s_%s' % (self.settings_prefix, attr),
                )
            except AttributeError:
                if not attr.startswith('DEFAULT_'):
                    setting = getattr(self, 'DEFAULT_%s' % attr)
                else:
                    raise

            return setting

        else:
            # Default behaviour
            raise AttributeError(
                'No setting or default available for \'%s\'' % attr
            )


class SudsDjangoCache(Cache):
    """
    Implement the suds cache interface using Django caching.
    Source: https://github.com/dpoirier/basket/blob/master/news/backends/exacttarget.py
    """
    def __init__(self, days=None, *args, **kwargs):
        if days:
            self.timeout = 24 * 60 * 60 * days
        else:
            self.timeout = None

    def _cache_key(self, id):
        return "suds-%s" % id

    def get(self, id):
        return cache.get(self._cache_key(id))

    def put(self, id, value):
        cache.set(self._cache_key(id), value, self.timeout)

    def purge(self, id):
        cache.delete(self._cache_key(id))


def import_object(from_path):
    """ Given an import path, return the object it represents. """

    module, attr = from_path.rsplit(".", 1)
    mod = import_module(module)
    return getattr(mod, attr)



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import requests

from django.core.exceptions import ImproperlyConfigured

from url_sso.utils import Singleton
from url_sso.settings import url_sso_settings
from url_sso.exceptions import RequestKeyException


class SSOPluginBase(object):
    """ Base class for URL SSO plugins. """

    __metaclass__ = Singleton

    def __init__(self):
        """ Setup a Requests session. """

        self.session = requests.Session()
        self.session.verify = True
        self.session.timeout = url_sso_settings.REQUEST_TIMEOUT

    def get_settings(self):
        """
        Utility method for obtaining plugin settings (such that they
        can be updated for tests). Uses the `settings_name` attribute on
        the plugin to find the settings.

        Example::

            class BananaPlugin(SSOPluginBase):
                settings_name = 'BANANA'

        In this case, get_settings() returns the URL_SSO_BANANA setting.
        """

        assert hasattr(self, 'settings_name'), \
            'Please set settings_name on the plugin.'
        assert self.settings_name.isupper(), \
            'Setting name should be upper case.'

        settings = getattr(url_sso_settings, self.settings_name, None)

        if not settings:
            raise ImproperlyConfigured(
                'Settings not available for %s.' % self.__class__
            )

        return settings

    def get_url(self, url, params={}):
        """
        Wrapper around requests.get() using sensible defaults.
        """

        try:
            r = self.session.get(
                url, params=params
            )
        except requests.exceptions.RequestException, e:
            # Raise exception, retaining original traceback
            traceback = sys.exc_info()[2]
            raise RequestKeyException('Error with HTTP request: %s' % e), \
                None, traceback

        return r



#!/usr/bin/env python
# This file is part of django-url-sso.
#
# django-url-sso: Generate login URL's for unstandardized SSO systems.
# Copyright (C) 2014 Mathijs de Bruin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" SSO for iProva http://www.infoland.nl/ """

import sys

import suds.client
import suds_requests

from django.core.cache import cache

from url_sso.exceptions import RequestKeyException
from url_sso.utils import SudsDjangoCache

from url_sso.plugins.base import SSOPluginBase


class iProvaPlugin(SSOPluginBase):
    settings_name = 'IPROVA'

    def _get_webservice(self):
        """ Return SOAP client (suds). """

        settings = self.get_settings()
        assert 'root_url' in settings
        root_url = settings['root_url']

        webservice_url = (
            root_url + 'Management/Webservices/UserManagementAPI.asmx?WSDL'
        )

        suds_cache = SudsDjangoCache()

        client = suds.client.Client(
            webservice_url,
            transport=suds_requests.RequestsTransport(self.session),
            cache=suds_cache,
            # Cache WSDL (pickled) object (not whole tokens)
            cachingpolicy=1
        )

        assert client.options.cache == suds_cache

        return client.service

    def _request_token(self, username):
        """ Request login token for a particular user. """

        settings = self.get_settings()
        assert 'application_id' in settings
        application_id = settings['application_id']

        assert isinstance(application_id, basestring)
        assert isinstance(username, basestring)

        try:
            webservice = self._get_webservice()

            result = webservice.GetTokenForUser(
                strTrustedApplicationID=application_id,
                strLoginCode=username
            )

        except Exception, e:
            # Raise exception, retaining original traceback
            traceback = sys.exc_info()[2]
            raise RequestKeyException('Error in SOAP request: %s' % e), \
                None, traceback

        return result

    def _get_cache_key(self, username):
        """ Return a sensible cache key for username. """
        return 'iprova_sso_{user}'.format(
            user=username
        )

    def _get_login_token(self, username):
        """ Return a valid (possibly cached) login token. """

        settings = self.get_settings()

        assert 'key_expiration' in settings

        cache_key = self._get_cache_key(username)
        cache_timeout = settings['key_expiration']

        token = cache.get(cache_key)
        if not token:
            # Request a new token
            token = self._request_token(username)

            cache.set(cache_key, token, cache_timeout)

        return token

    def _generate_login_url(self, url, token):
        """ Generate a login URL using supplied token. """

        settings = self.get_settings()
        assert 'root_url' in settings
        assert url.startswith(settings['root_url'])
        assert '?' not in url

        return '{0}?token={1}'.format(url, token)

    def get_login_urls(self, request):
        """ Return login URL """

        settings = self.get_settings()
        assert 'root_url' in settings
        assert 'services' in settings

        # Method to determine access for request (user), service
        has_access = settings.get(
            'has_access', lambda request, service: True
        )

        login_urls = {}

        if request.user.is_authenticated():

            # Start out with empty token to prevent requesting data for
            # users without permission
            token = None

            for service in settings['services']:
                # Determine whether user has access rights
                if not has_access(request, service):
                    # has_access(request) is defined and user does not have
                    # access to this site - skip.
                    continue

                if not token:
                    # Get token
                    token = self._get_login_token(request.user.username)

                # Generate key, e.g. 'IPROVA_MANAGEMENT_SSO_URL'
                url_key = '{0}_{1}_SSO_URL'.format(
                    self.settings_name, service.upper()
                )

                url = '{0}{1}/'.format(
                    settings['root_url'], service
                )

                login_urls[url_key] = self._generate_login_url(url, token)

        return login_urls


# Instantiate singleton
iprova_plugin = iProvaPlugin()
